const path = require('path')
const { reporter, chalk, exit } = require('@openhat/cli-helpers')
const FormData = require('form-data')
const fs = require('fs-extra')
const inquirer = require('inquirer')
const { constructAppUrl } = require('../lib/constructAppUrl')
const finalArchivePath = require('../lib/finalArchivePath.js')
const { createClient } = require('../lib/httpClient')
const parseConfig = require('../lib/parseConfig')
const makePaths = require('../lib/paths.js')

const dumpHttpError = (message, response) => {
    if (!response) {
        reporter.error(message)
        return
    }

    reporter.error(
        message,
        response.status,
        typeof response.data === 'object'
            ? response.data.message
            : response.statusText
    )
    reporter.debugErr('Error details', response.data)
}

const promptForOhimsConfig = async (params) => {
    if (
        process.env.CI &&
        (!params.baseUrl || !params.username || !params.password)
    ) {
        exit(
            1,
            'Prompt disabled in CI mode - missing baseUrl, username, or password parameter.'
        )
    }

    const isValidUrl = (input) =>
        input && input.length && input.match(/^https?:\/\/[^/.]+/)

    const responses = await inquirer.prompt([
        {
            type: 'input',
            name: 'baseUrl',
            message: 'OHIMS instance URL:',
            when: () => !params.baseUrl,
            validate: (input) =>
                isValidUrl(input)
                    ? true
                    : 'Please enter a valid URL, it must start with http:// or https://',
        },
        {
            type: 'input',
            name: 'username',
            message: 'OHIMS instance username:',
            when: () => !params.username,
        },
        {
            type: 'password',
            name: 'password',
            message: 'OHIMS instance password:',
            when: () => !params.password,
        },
    ])

    return {
        baseUrl: responses.baseUrl || params.baseUrl,
        auth: {
            username: responses.username || params.username,
            password: responses.password || params.password,
        },
    }
}

const handler = async ({ cwd = process.cwd(), timeout, ...params }) => {
    const paths = makePaths(cwd)
    const config = parseConfig(paths)

    const ohimsConfig = await promptForOhimsConfig(params)

    if (config.standalone) {
        exit(1, `Standalone apps cannot be deployed to OHIMS instances`)
    }

    const appBundle = path.relative(
        cwd,
        finalArchivePath({
            filepath: paths.buildAppBundle,
            name: config.name,
            version: config.version,
        })
    )

    if (!fs.existsSync(appBundle)) {
        exit(
            1,
            `App bundle does not exist, run ${chalk.bold(
                'oh-app-scripts build'
            )} before deploying.`
        )
    }

    const client = createClient(ohimsConfig)
    const formData = new FormData()
    formData.append('file', fs.createReadStream(appBundle))

    let serverVersion
    try {
        reporter.print(`Pinging server ${ohimsConfig.baseUrl}...`)
        const rawServerVersion = (await client.get('/api/system/info.json'))
            .data.version
        const parsedServerVersion = /(\d+)\.(\d+)/.exec(rawServerVersion)
        if (!parsedServerVersion) {
            exit(
                1,
                `Invalid server version ${rawServerVersion} found, aborting...`
            )
        }
        serverVersion = {
            full: parsedServerVersion[0],
            major: parsedServerVersion[1],
            minor: parsedServerVersion[2],
        }
        reporter.debug(
            'Found server version',
            serverVersion.full,
            `(${rawServerVersion})`
        )
    } catch (e) {
        dumpHttpError(
            `Server ${chalk.bold(ohimsConfig.baseUrl)} could not be contacted`,
            e.response
        )
        exit(1)
    }

    const appUrl = constructAppUrl(ohimsConfig.baseUrl, config, serverVersion)

    try {
        reporter.print('Uploading app bundle...')
        await client.post('/api/apps', formData, {
            headers: {
                ...formData.getHeaders(),
            },
            timeout: timeout * 1000, // Ensure we have enough time to upload a large zip file
        })
        reporter.info(
            `Successfully deployed ${config.name} to ${ohimsConfig.baseUrl}`
        )
    } catch (e) {
        dumpHttpError('Failed to upload app, HTTP error', e.response)
        exit(1)
    }

    reporter.debug(`Testing app launch url at ${appUrl}...`)
    try {
        await client.get(appUrl)
    } catch (e) {
        dumpHttpError(`Uploaded app not responding at ${appUrl}`)
        exit(1)
    }
    reporter.print(`App is available at ${appUrl}`)
}

const command = {
    command: 'deploy [baseUrl]',
    alias: 'd',
    desc: 'Deploy the built application to a specific OHIMS instance',
    builder: {
        username: {
            alias: 'u',
            description:
                'The username for authenticating with the OHIMS instance',
        },
        timeout: {
            description:
                'The timeout (in seconds) for uploading the app bundle',
            default: 300,
        },
    },
    handler,
}

module.exports = command
