const fs = require('fs')
const { reporter, prompt } = require('@openhat/cli-helpers')
const { validateLockfile } = require('./validateLockfile')

jest.mock('fs')
jest.mock('@openhat/cli-helpers')

describe('validateLockfile', () => {
    afterEach(() => {
        jest.clearAllMocks()
    })

    const mockYarnLock = (yarnLock) => {
        fs.readFileSync.mockReturnValue(yarnLock)
    }

    it('returns true if there are no duplicates', async () => {
        mockYarnLock(`# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"some-dependency":
  version "1.0.0"

"other-dependency":
  version "1.0.0"
`)

        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: false,
            })
        ).toBe(true)
        expect(reporter.warn).toHaveBeenCalledTimes(0)
    })

    it('returns true if duplicates are solely for non-sensitive dependencies', async () => {
        mockYarnLock(`# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"some-dependency":
  version "1.0.0"

"some-dependency":
  version "1.2.0"

"some-dependency":
  version "2.0.0"
`)

        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: false,
            })
        ).toBe(true)
        expect(reporter.warn).toHaveBeenCalledTimes(0)
    })

    it('detects minor version duplicates of sensitive dependencies', async () => {
        mockYarnLock(`# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"@openhat/ui@1.0.0":
  version "1.0.0"

"@openhat/ui@1.0.0":
  version "1.0.0"
`)

        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: false,
            })
        ).toBe(false)
        expect(reporter.warn).toHaveBeenCalledTimes(1)
        expect(reporter.warn).toHaveBeenLastCalledWith(
            `Found 2 versions of '@openhat/ui' in yarn.lock: 6.1.2, 1.0.0`
        )
    })

    it('detects mjaor version duplicates of sensitive dependencies', async () => {
        mockYarnLock(`# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"@openhat/ui@1.0.0":
  version "1.0.0"

"@openhat/ui@1.0.0":
  version "1.0.0"
`)

        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: false,
            })
        ).toBe(false)
        expect(reporter.warn).toHaveBeenCalledTimes(1)
        expect(reporter.warn).toHaveBeenLastCalledWith(
            `Found 2 versions of '@openhat/ui' in yarn.lock: 7.0.0, 1.0.0`
        )
    })

    it('only outputs warnings for sensitive dependencies', async () => {
        mockYarnLock(`# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"foo@^1.4.5":
  version "1.4.5"

"foo@1.2.3":
  version "1.2.3"
  dependencies:
    "@openhat/ui" "^6.25.0"

"@openhat/ui@^6.23.0":
  version "6.23.0"

"@openhat/ui@^6.25.0":
  version "6.25.0"

"@openhat/ui@^1.0.0":
  version "1.0.0"
`)

        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: false,
            })
        ).toBe(false)
        expect(reporter.warn).toHaveBeenCalledTimes(1)
        expect(reporter.warn).toHaveBeenLastCalledWith(
            `Found 3 versions of '@openhat/ui' in yarn.lock: 6.23.0, 6.25.0, 7.2.1`
        )
    })

    it('can fix lockfiles with duplicate minor versions', async () => {
        mockYarnLock(`# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"@openhat/ui@^6.0.0":
  version "6.0.0"

"@openhat/ui@^6.2.3":
  version "6.2.3"

"@openhat/ui@^6.1.2":
  version "6.1.2"
`)

        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: false,
            })
        ).toBe(false)
        expect(reporter.warn).toHaveBeenCalledTimes(1)
        expect(prompt).toHaveBeenCalledTimes(0)

        reporter.warn.mockClear()
        prompt.mockResolvedValue({ fix: false })
        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: true,
            })
        ).toBe(false)
        expect(reporter.warn).toHaveBeenCalledTimes(1)
        expect(prompt).toHaveBeenCalledTimes(1)

        reporter.warn.mockClear()
        prompt.mockClear()
        prompt.mockResolvedValue({ fix: true })
        expect(
            await validateLockfile(null, {
                paths: { yarnLock: 'yarn.lock' },
                offerFix: true,
            })
        ).toBe(true)
        expect(reporter.warn).toHaveBeenCalledTimes(1)
        expect(prompt).toHaveBeenCalledTimes(1)
        expect(fs.writeFileSync).toHaveBeenCalledTimes(1)
        expect(fs.writeFileSync).toHaveBeenLastCalledWith(
            'yarn.lock',
            `# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


"@openhat/ui@^6.0.0", "@openhat/ui@^6.1.2", "@openhat/ui@^6.2.3":
  version "6.2.3"
`
        )
    })
})
